@@ -37,7 +37,7 @@
 /* .IP \(bu
 /*	To provide single-updater functionality for lookup tables
 /*	that do not reliably support multiple writers (i.e. all
-/*	file-based tables).
+/*	file-based tables that are not based on \fBlmdb\fR).
 /* .PP
 /*	The \fBproxymap\fR(8) server implements the following requests:
 /* .IP "\fBopen\fR \fImaptype:mapname flags\fR"
@@ -215,8 +215,6 @@
 /*	Google, Inc.
 /*	111 8th Avenue
 /*	New York, NY 10011, USA
-/*
-/*	Wietse Venema
 /*--*/
 
 /* System library. */
@@ -687,13 +685,13 @@ DICT   *dict_proxy_open(const char *map, int open_flags, int dict_flags)
 
 /* authorize_proxied_maps - recursively authorize maps */
 
-static void authorize_proxied_maps(char *bp, const char *blame)
+static void authorize_proxied_maps(char *bp)
 {
     const char *sep = CHARS_COMMA_SP;
     const char *parens = CHARS_BRACE;
     char   *type_name;
 
-    while ((type_name = mystrtokq_cw(&bp, sep, parens, blame)) != 0) {
+    while ((type_name = mystrtokq(&bp, sep, parens)) != 0) {
 	char   *nested_info;
 
 	/* Maybe { maptype:mapname attr=value... } */
@@ -708,7 +706,7 @@ static void authorize_proxied_maps(char *bp, const char *blame)
 		continue;
 	    }
 	    /* Don't try to second-guess the semantics of { }. */
-	    if ((type_name = mystrtokq_cw(&type_name, sep, parens, blame)) == 0)
+	    if ((type_name = mystrtokq(&type_name, sep, parens)) == 0)
 		continue;
 	}
 	/* Recurse into nested map (pipemap, unionmap). */
@@ -724,7 +722,7 @@ static void authorize_proxied_maps(char *bp, const char *blame)
 		myfree(err);
 		continue;
 	    }
-	    authorize_proxied_maps(nested_info, blame);
+	    authorize_proxied_maps(nested_info);
 	    continue;
 	}
 	if (strncmp(type_name, PROXY_COLON, PROXY_COLON_LEN))
@@ -754,8 +752,10 @@ static void post_jail_init(char *service_name, char **unused_argv)
     if (strcmp(service_name, MAIL_SERVICE_PROXYWRITE) == 0)
 	proxy_writer = 1;
     else if (strcmp(service_name, MAIL_SERVICE_PROXYMAP) != 0)
-	msg_fatal("service name must be one of %s or %s",
-		  MAIL_SERVICE_PROXYMAP, MAIL_SERVICE_PROXYMAP);
+	msg_fatal("invalid service name: \"%s\" - "
+		  "service name must be \"%s\" or \"%s\"",
+		  service_name, MAIL_SERVICE_PROXYWRITE,
+		  MAIL_SERVICE_PROXYMAP);
 
     /*
      * Pre-allocate buffers.
@@ -772,8 +772,7 @@ static void post_jail_init(char *service_name, char **unused_argv)
     saved_filter = mystrdup(proxy_writer ? var_proxy_write_maps :
 			    var_proxy_read_maps);
     proxy_auth_maps = htable_create(13);
-    authorize_proxied_maps(saved_filter, proxy_writer ? VAR_PROXY_WRITE_MAPS :
-			   VAR_PROXY_READ_MAPS);
+    authorize_proxied_maps(saved_filter);
     myfree(saved_filter);
 
     /*
@@ -844,6 +843,44 @@ int     main(int argc, char **argv)
      */
     MAIL_VERSION_STAMP_ALLOCATE;
 
+    /*
+     * Workaround for programs that make explicit dict_register() calls with
+     * a table that is already registered under a different name. This is
+     * safe only in programs that do not unregister or close a table that is
+     * registered with multiple names.
+     */
+    dict_allow_multiple_dict_register_names = 1;
+
+    /*
+     * XXX When invoked with the master.cf service name "proxywrite", the
+     * proxymap daemon will allow update requests. To update a table that is
+     * not multi-writer safe (for example, some versions of Berkeley DB), the
+     * "proxywrite" service should run as a single updater (i.e. a process
+     * limit of 1, which could be enforced below by requesting
+     * CA_MAIL_SERVER_SOLITARY).
+     * 
+     * In the default master.cf file, the "proxywrite" service has a process
+     * limit of 1. Assuming that updates will be rare, this process limit
+     * will suffice. Latency-sensitive services such as postscreen must not
+     * use the proxywrite service (in fact, postscreen has a latency check
+     * built-in).
+     * 
+     * Optimizing for multi-writer operation would suffer from all kinds of
+     * complexity that would make it hard to use:
+     * 
+     * - The master daemon specifies the "proxywrite" service name with the -n
+     * command-line option. This information is not known here, before the
+     * multi_server_main() call. The multi_server_main() function could
+     * reveal process limit information to its call-back functions, and leave
+     * single-updater enforcement to its call-back functions.
+     * 
+     * - If we really want multi-writer update support, the "proxywrite" service
+     * would have to parse the $proxy_write_maps value, and permit
+     * multi-writer operation only if all tables are multi-writer safe. That
+     * would require a new dict(3) method, to query each lookup table
+     * implementation if it is multi-writer safe, without instantiating a
+     * lookup table client.
+     */
     multi_server_main(argc, argv, proxymap_service,
 		      CA_MAIL_SERVER_STR_TABLE(str_table),
 		      CA_MAIL_SERVER_POST_INIT(post_jail_init),
