@@ -406,7 +406,7 @@ static void dict_node_free(void *ptr)
     DICT_NODE *node = (DICT_NODE *) ptr;
     DICT   *dict = node->dict;
 
-    if (dict->saved_close)			/* managed by dict_open() */
+    if (dict->saved_close)			/* managed by dict_register() */
 	dict->saved_close(dict);
     else
 	dict->close(dict);
